from mixin import Mixin
from typing import Union, List


class Content:
    """Content é o elemento que recebe o AdaptiveCard e é adicionado à lista atachments, atributo de Message"""
    def __init__(self, content: "AdaptiveCard"):
        self.contentType = "application/vnd.microsoft.card.adaptive"
        self.content = content

class Message(Mixin):
    """"Estrutura final do card tal como se requer para envio a um canal do Teams"""
    def __init__(self, attachments: Union[None, List["Content"]] = None):
        self.type = "message"
        if attachments == None:
            attachments = []
        self.attachments = attachments

    def attach(self, content):
        self.attachments.append(content)

#-------------------------Cards-------------------------#

class AdaptiveCard(Mixin):
    """O template principal do card"""  # Essas descrições hão de ficar mais detalhadas à medida que eu desenvolver a lib e sua documentação
    def __init__(self, type: str = "AdaptiveCard", version = "1.2", schema: str = "http://adaptivecards.io/schemas/adaptive-card.json", body: Union[None, List[Union["ColumnSet", "Container", "TextBlock", "Image"]]] = None, falbackText: Union[None, str] = None, backgroundImage = None, minHeight = None, rtl = None, speak = None, lang = None, verticalContentAlignment = None):
        self.type = type
        self.version = version
        super().set_version(self.version)
        self.schema = schema

        if body == None:
            body = []

        self.body = body
        super().create_fields(locals())

    def add_to_body(self, card_element):
        self.body.append(card_element)

#---------------------Containers-------------------------#

class Container(Mixin):
    """Um contâiner é um agrupamento de elementos"""
    def __init__(self, items = None, style: Union[None, str] = None, verticalContentAlignment: Union[None, str] = None, bleed: Union[None, bool] = None, minHeight: Union[None, str] = None, rtl: Union[None, bool] = None):
        self.type = "Container"

        if items == None:
            items = []
        self.items = items

        super().create_fields(locals())

    def add_to_items(self, card_element):
        self.items.append(card_element)

class ColumnSet(Mixin):
    """ColumnSet define um grupo de colunas"""
    def __init__(self, columns: Union[None, List["Column"]] = None, style: Union[None, str] = None, bleed: Union[None, bool] = None,
                 minHeight: Union[None, str] = None, horizontalAlignment: Union[None, str] = None):
        self.type = "ColumnSet"
        if columns is None:
            columns = []
        self.columns = columns

        super().create_fields(locals())

    def add_to_columns(self, column_element):
        self.columns.append(column_element)

class Column(Mixin):
    """O contâiner Column define um elemento de coluna, que é parte de um ColumnSet."""
    def __init__(self, items: Union[None, List[Union["Image", "TextBlock"]]] = None, backgroundImage = None, bleed: Union[None, bool] = None, fallback = None,
                 minHeight: Union[None, str] = None, rtl: Union[None, bool] = None, separator: Union[None, bool] = None,
                 spacing: Union[None, str, int] = None, style: Union[None, str] = None, verticalContentAlignment: Union[None, str] = None,
                 width: Union[None, str, int] = None, id: Union[None, str] = None, isVisible: Union[None, bool] = None):
        self.type = "Column"
        if items is None:
            items = []

        super().create_fields(locals())

    def add_to_items(self, card_element):
        self.items.append(card_element)

class Table(Mixin):
    def __init__(self, columns: Union[None, List[dict]], rows: Union[None, List[dict]], firstRowAsHeader: Union[None, bool], showGridLines: Union[None, bool],
    gridStyle: Union[None, str], horizontalCellContentAlignment = Union[None, str], verticalCellContentAlignment: Union[None, str] = None,
    fallback: Union[None, "ColumnSet", "Container", "Image", "Table"] = None, height: Union[None, str] = None, separator: Union[None, bool] = None,
    spacing: Union[None, str] = None, id: Union[None, str] = None, isVisible = Union[None, bool]):
        self.type = "Table"

class TableCell(Mixin):
    def __init__(self, items: Union[None, "ColumnSet", "Container", "Image", "Table"] = None, selectAction: None = None, style: Union[None, str] = None,
    verticalContentAlignment: Union[None, str] = None, bleed: Union[None, bool] = None, backgroundImage: Union[None, str] = None,
    minHeight: Union[None, str] = None, rtl: Union[None, bool] = None):
        pass
#-------------------------Card Elements---------------------------#

class TextBlock(Mixin):
    """Elemento de texto"""
    def __init__(self, text: str = "", color: Union[None, str] = None, fontType: Union[None, str] = None, horizontalAlignment: Union[None, str] = None,
                isSubtle: Union[None, bool] = None, maxLines: Union[None, int] = None, size: Union[None, str] = None, weight: Union[None, str] = None,
                wrap: Union[None, bool] = None, style: Union[None, str] = None, fallback: Union[None, str, ColumnSet, "Container", "TextBlock"] = None, height: Union[None, str] = None,
                separator: Union[None, bool] = None, spacing: Union[None, str] = None, id: Union[None, str] = None, isVisible: Union[None, bool] = None):
        self.type = "TextBlock"
        self.text = text
        super().create_fields(locals())
    def update_text(self, text):
        self.text = text

class Image(Mixin):
    def __init__(self, url: str, altText: Union[None, str] = None, backgroundColor: Union[None, str] = None, height: Union[None, str] = None, horizontalAlignment: Union[None, str] = None, selectAction: Union[None, str] = None):
        self.type = "Image"
        super().create_fields(locals())
    


import json
# from typing import get_type_hints, Union
# from typing_extensions import get_args, get_origin


class Mixin:    # essa é uma classe do tipo mixin. Não tem funcionalidade própria, serve para fornecer funcionalidade a outras classes.
    def create_fields(self, local_variables_dict) -> None:
        variables = [parameter for parameter in self.__init__.__code__.co_varnames if parameter != "self"]
        for variable in variables:
            value = local_variables_dict[variable]
            if value is not None:
                self.__dict__[variable] = value

    # def create_fields(self, local_variables_dict) -> None:
    #     variables = get_type_hints(self.__init__)
    #     for variable in variables:
    #         value = local_variables_dict[variable]
    #         expected_type = variables[variable]
    #         if value is not None:
    #             type_origin = get_origin(expected_type)
    #             if type_origin is None:
    #                 if type(value) != expected_type:
    #                     msg = "'{variable}' attribute cannot be of type '{data_type}'"
    #                     raise Exception(msg)
    #             elif type_origin == list:
    #                 if type(value) != list:
    #                     msg = ""
    #                     raise Exception(msg)
    #                 else:
    #                     expected_types_tuple = get_args(expected_type)
    #                     type_origins_list = [x if get_origin(x) is None else get_origin(x) for x in expected_types_tuple]
    #                     expected_types_inside_list = get_args(expected_types_tuple[type_origins_list.index(list)])
    #                     if get_origin(expected_types_inside_list)[0] == Union:
    #                         expected_types_inside_list = list(get_args(expected_types_inside_list[0]))
    #                     for i in range(len(expected_types_inside_list)):
    #                         if isinstance(expected_types_inside_list[i], str):
    #                             expected_types_inside_list[i] == eval(expected_types_inside_list[i])
    #                     for x in value:
    #                         if type(x) not in expected_types_inside_list:
    #                             msg = f"objects of type '{type(x)}' are not allowed in '{variable}' list"
    #                             raise Exception(msg)
    #             elif type_origin == Union:
    #                 expected_types_tuple = get_args(expected_type)
    #                 type_origins_list = [x if get_origin(x) is None else get_origin(x) for x in expected_types_tuple]
    #                 if type(value) not in type_origins_list:
    #                     msg = f"'{variable}' attribute cannot be of type {type(value)}"
    #                     raise Exception(msg)
    #                 elif type(value) == list:
    #                     expected_types_inside_list = get_args(expected_types_tuple[type_origins_list.index(list)])
    #                     if get_origin(expected_types_inside_list[0]) == Union:
    #                         expected_types_inside_list = list(get_args(expected_types_inside_list[0]))
    #                     for x in value:
    #                         if type(x) not in expected_types_inside_list:
    #                             msg = f"objects of type '{type(x)}' are not allowed in '{variable}' list"
    #                             raise Exception(msg)

    #             self.__dict__[variable] = value

    def to_dict(self):
        dic = json.loads(json.dumps(self, default=lambda obj: obj.__dict__))  # isso é um hack muito ixpertinho
        return dic

    def set_version(self, version: str) -> None:
        self.version = version



from re import L
from setuptools import setup, find_packages
import codecs
import os

VERSION = '0.0.3'
DESCRIPTION = 'Microsoft Adaptive Cards'
LONG_DESCRIPTION = 'A package that helps you create adaptive cards.'

setup(
    name="adaptivecard",
    version=VERSION,
    author="cabutchei (Luan Paz)",
    author_email="<luropa_paz@hotmail.com>",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    keywords=['python', 'card', 'adaptive card', 'microsoft'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)